@@ -26,8 +26,12 @@
 #include <unistd.h>
 #include <signal.h>
 
-#include <xcb/xtest.h>
+#include <xcb/bigreq.h>
+#include <xcb/randr.h>
+#include <xcb/shape.h>
 #include <xcb/xcb_event.h>
+#include <xcb/xinerama.h>
+#include <xcb/xtest.h>
 
 #include "awesome.h"
 #include "spawn.h"
@@ -69,14 +73,6 @@ awesome_atexit(void)
 
     a_dbus_cleanup();
 
-    /* reparent systray windows, otherwise they may die with their master */
-    for(int i = 0; i < globalconf.embedded.len; i++)
-    {
-        xcb_screen_t *s = xutil_screen_get(globalconf.connection,
-                                           globalconf.embedded.tab[i].phys_screen);
-        xembed_window_unembed(globalconf.connection, globalconf.embedded.tab[i].win, s->root);
-    }
-
     /* do this only for real screen */
     const xcb_setup_t *setup = xcb_get_setup(globalconf.connection);
     nscreens = setup ? xcb_setup_roots_length(setup) : -1;
@@ -85,13 +81,6 @@ awesome_atexit(void)
         screen_nbr++)
         systray_cleanup(screen_nbr);
 
-    /* remap all clients since some WM won't handle them otherwise */
-    foreach(c, globalconf.clients)
-    {
-        client_unban(*c);
-        titlebar_client_detach(*c);
-    }
-
     /* Close Lua */
     lua_close(globalconf.L);
 
@@ -220,14 +209,14 @@ a_xcb_check_cb(EV_P_ ev_check *w, int revents)
         }
         else
         {
-            xcb_event_handle(&globalconf.evenths, event);
+            event_handle(event);
             p_delete(&event);
         }
     }
 
     if(mouse)
     {
-        xcb_event_handle(&globalconf.evenths, mouse);
+        event_handle(mouse);
         p_delete(&mouse);
     }
 }
@@ -235,21 +224,10 @@ a_xcb_check_cb(EV_P_ ev_check *w, int revents)
 static void
 a_xcb_io_cb(EV_P_ ev_io *w, int revents)
 {
-    /* empty */
-}
+    /* a_xcb_check_cb() already handled all events */
 
-/** Startup Error handler to check if another window manager
- * is already running.
- * \param data Additional optional parameters data.
- * \param c X connection.
- * \param error Error event.
- */
-static int __attribute__ ((noreturn))
-xerrorstart(void * data __attribute__ ((unused)),
-            xcb_connection_t * c  __attribute__ ((unused)),
-            xcb_generic_error_t * error __attribute__ ((unused)))
-{
-    fatal("another window manager is already running");
+    if(xcb_connection_has_error(globalconf.connection))
+        fatal("X server connection broke");
 }
 
 static void
@@ -287,37 +265,6 @@ restart_on_signal(EV_P_ ev_signal *w, int revents)
     awesome_restart();
 }
 
-/** \brief awesome xerror function.
- * There's no way to check accesses to destroyed windows, thus those cases are
- * ignored (especially on UnmapNotify's).  Other types of errors call Xlibs
- * default error handler, which may call exit.
- * \param data Currently unused.
- * \param c The connection to the X server.
- * \param e The error event.
- * \return 0 if no error, or xerror's xlib return status.
- */
-static int
-xerror(void *data __attribute__ ((unused)),
-       xcb_connection_t *c __attribute__ ((unused)),
-       xcb_generic_error_t *e)
-{
-    /* ignore this */
-    if(e->error_code == XCB_EVENT_ERROR_BAD_WINDOW
-       || (e->error_code == XCB_EVENT_ERROR_BAD_MATCH
-           && e->major_code == XCB_SET_INPUT_FOCUS)
-       || (e->error_code == XCB_EVENT_ERROR_BAD_VALUE
-           && e->major_code == XCB_KILL_CLIENT)
-       || (e->major_code == XCB_CONFIGURE_WINDOW
-           && e->error_code == XCB_EVENT_ERROR_BAD_MATCH))
-        return 0;
-
-    warn("X error: request=%s, error=%s",
-         xcb_event_get_request_label(e->major_code),
-         xcb_event_get_error_label(e->error_code));
-
-    return 0;
-}
-
 /** Print help and exit(2) with given exit_code.
  * \param exit_code The exit code.
  */
@@ -444,10 +391,12 @@ main(int argc, char **argv)
     if(xcb_connection_has_error(globalconf.connection))
         fatal("cannot open display");
 
-    /* check for xtest extension */
-    const xcb_query_extension_reply_t *xtest_query;
-    xtest_query = xcb_get_extension_data(globalconf.connection, &xcb_test_id);
-    globalconf.have_xtest = xtest_query->present;
+    /* Prefetch all the extensions we might need */
+    xcb_prefetch_extension_data(globalconf.connection, &xcb_big_requests_id);
+    xcb_prefetch_extension_data(globalconf.connection, &xcb_test_id);
+    xcb_prefetch_extension_data(globalconf.connection, &xcb_randr_id);
+    xcb_prefetch_extension_data(globalconf.connection, &xcb_xinerama_id);
+    xcb_prefetch_extension_data(globalconf.connection, &xcb_shape_id);
 
     /* initialize dbus */
     a_dbus_init();
@@ -467,10 +416,6 @@ main(int argc, char **argv)
     ev_prepare_start(globalconf.loop, &a_refresh);
     ev_unref(globalconf.loop);
 
-    /* Allocate a handler which will holds all errors and events */
-    xcb_event_handlers_init(globalconf.connection, &globalconf.evenths);
-    xutil_error_handler_catch_all_set(&globalconf.evenths, xerrorstart, NULL);
-
     for(screen_nbr = 0;
         screen_nbr < xcb_setup_roots_length(xcb_get_setup(globalconf.connection));
         screen_nbr++)
@@ -486,11 +431,18 @@ main(int argc, char **argv)
     /* Need to xcb_flush to validate error handler */
     xcb_aux_sync(globalconf.connection);
 
-    /* Process all errors in the queue if any */
-    xcb_event_poll_for_event_loop(&globalconf.evenths);
+    /* Process all errors in the queue if any. There can be no events yet, so if
+     * this function returns something, it must be an error. */
+    if (xcb_poll_for_event(globalconf.connection) != NULL)
+        fatal("another window manager is already running");
 
-    /* Set the default xerror handler */
-    xutil_error_handler_catch_all_set(&globalconf.evenths, xerror, NULL);
+    /* Prefetch the maximum request length */
+    xcb_prefetch_maximum_request_length(globalconf.connection);
+
+    /* check for xtest extension */
+    const xcb_query_extension_reply_t *xtest_query;
+    xtest_query = xcb_get_extension_data(globalconf.connection, &xcb_test_id);
+    globalconf.have_xtest = xtest_query->present;
 
     /* Allocate the key symbols */
     globalconf.keysyms = xcb_key_symbols_alloc(globalconf.connection);
@@ -559,11 +511,6 @@ main(int argc, char **argv)
     /* scan existing windows */
     scan();
 
-    /* process all errors in the queue if any */
-    xcb_event_poll_for_event_loop(&globalconf.evenths);
-    a_xcb_set_event_handlers();
-    a_xcb_set_property_handlers();
-
     /* we will receive events, stop grabbing server */
     xcb_ungrab_server(globalconf.connection);
     xcb_flush(globalconf.connection);
