@@ -301,7 +301,7 @@ client_unfocus(client_t *c)
      * the WM_TAKE_FOCUS protocol.
      */
     xcb_set_input_focus(globalconf.connection, XCB_INPUT_FOCUS_PARENT,
-                        root_win, XCB_CURRENT_TIME);
+                        root_win, globalconf.timestamp);
 
     client_unfocus_update(c);
 }
@@ -330,7 +330,7 @@ client_set_focus(client_t *c, bool set_input_focus)
     bool takefocus = client_hasproto(c, WM_TAKE_FOCUS);
     if(set_input_focus)
         xcb_set_input_focus(globalconf.connection, XCB_INPUT_FOCUS_PARENT,
-                            c->window, XCB_CURRENT_TIME);
+                            c->window, globalconf.timestamp);
     if(takefocus)
         window_takefocus(c->window);
 }
@@ -409,17 +409,13 @@ client_focus_update(client_t *c)
             /* Already focused */
             return;
     }
-    luaA_object_push(globalconf.L, c);
-    client_set_minimized(globalconf.L, -1, false);
-
-    /* unban the client before focusing for consistency */
-    client_unban(c);
 
     globalconf.screen_focus = &globalconf.screens.tab[c->phys_screen];
     globalconf.screen_focus->prev_client_focus = c;
     globalconf.screen_focus->client_focus = c;
 
     /* according to EWMH, we have to remove the urgent state from a client */
+    luaA_object_push(globalconf.L, c);
     client_set_urgent(globalconf.L, -1, false);
 
     ewmh_update_net_active_window(c->phys_screen);
@@ -447,6 +443,9 @@ client_focus(client_t *c)
     if(!client_maybevisible(c, c->screen))
         return;
 
+    /* X11 doesn't let you focus a window that isn't viewable */
+    client_unban(c);
+
     if (!c->nofocus)
         client_focus_update(c);
 
@@ -516,7 +515,7 @@ client_layer_translator(client_t *c)
     /* first deal with user set attributes */
     if(c->ontop)
         return LAYER_ONTOP;
-    else if(c->fullscreen)
+    else if(c->fullscreen && globalconf.screen_focus->client_focus == c)
         return LAYER_FULLSCREEN;
     else if(c->above)
         return LAYER_ABOVE;
@@ -620,11 +619,22 @@ client_manage(xcb_window_t w, xcb_get_geometry_reply_t *wgeom, int phys_screen,
      * startup id. */
     xcb_get_property_cookie_t startup_id_q = { 0 };
     if(!startup)
-        startup_id_q = xcb_get_any_property(globalconf.connection,
-                                            false, w, _NET_STARTUP_ID, UINT_MAX);
+        startup_id_q = xcb_get_property(globalconf.connection, false, w,
+                                        _NET_STARTUP_ID, XCB_GET_PROPERTY_TYPE_ANY, 0, UINT_MAX);
 
     xcb_change_window_attributes(globalconf.connection, w, XCB_CW_EVENT_MASK, select_input_val);
 
+    /* Make sure the window is automatically mapped if awesome exits or dies. */
+    xcb_change_save_set(globalconf.connection, XCB_SET_MODE_INSERT, w);
+
+    /* Move this window to the bottom of the stack. Without this we would force
+     * other windows which will be above this one to redraw themselves because
+     * this window occludes them for a tiny moment. The next stack_refresh()
+     * will fix this up and move the window to its correct place. */
+    xcb_configure_window(globalconf.connection, w,
+                         XCB_CONFIG_WINDOW_STACK_MODE,
+                         (uint32_t[]) { XCB_STACK_MODE_BELOW});
+
     client_t *c = client_new(globalconf.L);
 
     /* This cannot change, ever. */
@@ -754,12 +764,15 @@ area_t
 client_geometry_hints(client_t *c, area_t geometry)
 {
     int32_t basew, baseh, minw, minh;
+    int32_t real_basew = 0, real_baseh = 0;
 
     /* base size is substituted with min size if not specified */
     if(c->size_hints.flags & XCB_SIZE_HINT_P_SIZE)
     {
         basew = c->size_hints.base_width;
         baseh = c->size_hints.base_height;
+        real_basew = basew;
+        real_baseh = baseh;
     }
     else if(c->size_hints.flags & XCB_SIZE_HINT_P_MIN_SIZE)
     {
@@ -784,31 +797,38 @@ client_geometry_hints(client_t *c, area_t geometry)
         minw = minh = 0;
 
     if(c->size_hints.flags & XCB_SIZE_HINT_P_ASPECT
-       && c->size_hints.min_aspect_num > 0
        && c->size_hints.min_aspect_den > 0
-       && geometry.height - baseh > 0
-       && geometry.width - basew > 0)
+       && c->size_hints.max_aspect_den > 0
+       && geometry.height - real_baseh > 0
+       && geometry.width - real_basew > 0)
     {
-        double dx = (double) (geometry.width - basew);
-        double dy = (double) (geometry.height - baseh);
+        /* ICCCM mandates:
+         * If a base size is provided along with the aspect ratio fields, the
+         * base size should be subtracted from the window size prior to checking
+         * that the aspect ratio falls in range. If a base size is not provided,
+         * nothing should be subtracted from the window size. (The minimum size
+         * is not to be used in place of the base size for this purpose.) */
+        double dx = (double) (geometry.width - real_basew);
+        double dy = (double) (geometry.height - real_baseh);
         double min = (double) c->size_hints.min_aspect_num / (double) c->size_hints.min_aspect_den;
-        double max = (double) c->size_hints.max_aspect_num / (double) c->size_hints.min_aspect_den;
+        double max = (double) c->size_hints.max_aspect_num / (double) c->size_hints.max_aspect_den;
         double ratio = dx / dy;
         if(max > 0 && min > 0 && ratio > 0)
         {
             if(ratio < min)
             {
-                dy = (dx * min + dy) / (min * min + 1);
-                dx = dy * min;
-                geometry.width = (int) dx + basew;
-                geometry.height = (int) dy + baseh;
+                /* dx is lower than allowed, make dy lower to compensate this
+                 * (+ 0.5 to force proper rounding). */
+                dy = dx / min + 0.5;
+                geometry.width = (int) dx + real_basew;
+                geometry.height = (int) dy + real_baseh;
             }
             else if(ratio > max)
             {
-                dy = (dx * min + dy) / (max * max + 1);
-                dx = dy * min;
-                geometry.width = (int) dx + basew;
-                geometry.height = (int) dy + baseh;
+                /* dx is too high, lower it (+0.5 for proper rounding) */
+                dx = dy * max + 0.5;
+                geometry.width = (int) dx + real_basew;
+                geometry.height = (int) dy + real_baseh;
             }
         }
     }
@@ -865,16 +885,22 @@ client_resize(client_t *c, area_t geometry, bool hints)
         geometry.y = 0;
 
     /* Real client geometry, please keep it contained to C code at the very least. */
-    geometry_internal = titlebar_geometry_remove(c->titlebar, c->border_width, geometry);
+    if (!c->fullscreen)
+        geometry_internal = titlebar_geometry_remove(c->titlebar, c->border_width, geometry);
+    else
+        geometry_internal = geometry;
 
-    if(hints)
+    if(hints && !c->fullscreen)
         geometry_internal = client_geometry_hints(c, geometry_internal);
 
     if(geometry_internal.width == 0 || geometry_internal.height == 0)
         return false;
 
     /* Also let client hints propagate to the "official" geometry. */
-    geometry = titlebar_geometry_add(c->titlebar, c->border_width, geometry_internal);
+    if (!c->fullscreen)
+        geometry = titlebar_geometry_add(c->titlebar, c->border_width, geometry_internal);
+    else
+        geometry = geometry_internal;
 
     if(c->geometries.internal.x != geometry_internal.x
        || c->geometries.internal.y != geometry_internal.y
@@ -1010,6 +1036,8 @@ client_set_fullscreen(lua_State *L, int cidx, bool s)
         }
         else
         {
+            /* The titlebar was banned, unban! */
+            titlebar_unban(c->titlebar);
             geometry = c->geometries.fullscreen;
             c->fullscreen = false;
             client_set_border_width(L, cidx, c->border_width_fs);
@@ -1235,6 +1263,11 @@ client_unban(client_t *c)
         xcb_map_window(globalconf.connection, c->window);
 
         c->isbanned = false;
+
+        /* An unbanned clients shouldn't be minimized */
+        luaA_object_push(globalconf.L, c);
+        client_set_minimized(globalconf.L, -1, false);
+        lua_pop(globalconf.L, 1);
     }
 }
 
@@ -1290,12 +1323,18 @@ client_unmanage(client_t *c)
     if(strut_has_value(&c->strut))
         screen_emit_signal(globalconf.L, c->screen, "property::workarea", 0);
 
-    window_state_set(c->window, XCB_WM_STATE_WITHDRAWN);
-
     titlebar_client_detach(c);
 
     ewmh_update_net_client_list(c->phys_screen);
 
+    /* Remove this window from the save set since this shouldn't be made visible
+     * after a restart anymore. */
+    xcb_change_save_set(globalconf.connection, XCB_SET_MODE_DELETE, c->window);
+
+    /* Do this last to avoid races with clients. According to ICCCM, clients
+     * arent allowed to re-use the window until after this. */
+    window_state_set(c->window, XCB_WM_STATE_WITHDRAWN);
+
     /* set client as invalid */
     c->invalid = true;
 
@@ -1319,7 +1358,7 @@ client_kill(client_t *c)
         ev.response_type = XCB_CLIENT_MESSAGE;
         ev.window = c->window;
         ev.format = 32;
-        ev.data.data32[1] = XCB_CURRENT_TIME;
+        ev.data.data32[1] = globalconf.timestamp;
         ev.type = WM_PROTOCOLS;
         ev.data.data32[0] = WM_DELETE_WINDOW;
 
@@ -1971,6 +2010,22 @@ luaA_client_get_titlebar(lua_State *L, client_t *c)
     return luaA_object_push(L, c->titlebar);
 }
 
+static int
+luaA_client_get_focusable(lua_State *L, client_t *c)
+{
+    bool ret;
+
+    /* A client can be focused if it doesnt have the "nofocus" hint...*/
+    if (!c->nofocus)
+        ret = true;
+    else
+        /* ...or if it knows the WM_TAKE_FOCUS protocol */
+        ret = client_hasproto(c, WM_TAKE_FOCUS);
+
+    lua_pushboolean(L, ret);
+    return 1;
+}
+
 static int
 luaA_client_get_size_hints(lua_State *L, client_t *c)
 {
@@ -2354,6 +2409,10 @@ client_class_setup(lua_State *L)
                             NULL,
                             (lua_class_propfunc_t) luaA_client_get_size_hints,
                             NULL);
+    luaA_class_add_property(&client_class, A_TK_FOCUSABLE,
+                            NULL,
+                            (lua_class_propfunc_t) luaA_client_get_focusable,
+                            NULL);
 }
 
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
