@@ -62,7 +62,7 @@ extern const struct luaL_reg awesome_screen_meta[];
  * \return The number of elements pushed on stack.
  */
 static int
-luaA_quit(lua_State *L __attribute__ ((unused)))
+luaA_quit(lua_State *L)
 {
     ev_unloop(globalconf.loop, 1);
     return 0;
@@ -89,7 +89,7 @@ luaA_exec(lua_State *L)
 /** Restart awesome.
  */
 static int
-luaA_restart(lua_State *L __attribute__ ((unused)))
+luaA_restart(lua_State *L)
 {
     awesome_restart();
     return 0;
@@ -862,8 +862,10 @@ luaA_loadrc(const char *confpath, bool run)
             }
         }
         else
+        {
             lua_pop(globalconf.L, 1);
-        return true;
+            return true;
+        }
     }
     else
         fprintf(stderr, "%s\n", lua_tostring(globalconf.L, -1));
