@@ -252,7 +252,7 @@ widget_render(wibox_t *wibox)
         xcb_get_property_cookie_t prop_c;
         xcb_screen_t *s = xutil_screen_get(globalconf.connection, ctx->phys_screen);
         prop_c = xcb_get_property_unchecked(globalconf.connection, false, s->root, _XROOTPMAP_ID,
-                                            PIXMAP, 0, 1);
+                                            XCB_ATOM_PIXMAP, 0, 1);
         if((prop_r = xcb_get_property_reply(globalconf.connection, prop_c, NULL)))
         {
             if(prop_r->value_len
@@ -304,8 +304,18 @@ widget_render(wibox_t *wibox)
     /* remove wibox */
     lua_remove(globalconf.L, -1);
 
+    size_t num_widgets = lua_objlen(L, -1);
+    if(lua_objlen(L, -1) != (size_t) widgets->len)
+    {
+        warn("Widget layout returned wrong number of geometries. Got %d widgets and %lu geometries.",
+                widgets->len, (unsigned long) lua_objlen(L, -1));
+        /* Set num_widgets to min(widgets->len, lua_objlen(L, -1)); */
+        if(num_widgets > (size_t) widgets->len)
+            num_widgets = widgets->len;
+    }
+
     /* get computed geometries */
-    for(unsigned int i = 0; i < lua_objlen(L, -1); i++)
+    for(size_t i = 0; i < num_widgets; i++)
     {
         lua_pushnumber(L, i + 1);
         lua_gettable(L, -2);
@@ -333,6 +343,8 @@ widget_render(wibox_t *wibox)
             widgets->tab[i].widget->draw(widgets->tab[i].widget,
                                          ctx, widgets->tab[i].geometry, wibox);
 
+    cairo_surface_flush(ctx->surface);
+
     switch(wibox->orientation)
     {
         case South:
